@@ -1,27 +1,22 @@
 //! The core lib of Savlo web server framework.
-
+//! Read more: <https://salvo.rs>
+#![doc(html_favicon_url = "https://salvo.rs/images/favicon-32x32.png")]
+#![doc(html_logo_url = "https://salvo.rs/images/logo.svg")]
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![deny(private_in_public, unreachable_pub)]
 #![forbid(unsafe_code)]
 #![warn(missing_docs)]
 
-/// catcher module
 pub mod catcher;
 mod depot;
 mod error;
-/// fs module
 pub mod fs;
 mod handler;
-/// http module
 pub mod http;
-/// routing module
 pub mod routing;
-mod server;
+pub mod server;
 mod service;
-#[cfg(feature = "tls")]
-mod tls;
 mod transport;
-/// writer module
 pub mod writer;
 
 #[cfg(feature = "anyhow")]
@@ -34,9 +29,11 @@ pub use self::error::Error;
 pub use self::handler::Handler;
 pub use self::http::{Request, Response};
 pub use self::routing::Router;
-pub use self::server::Server;
 #[cfg(feature = "tls")]
-pub use self::server::TlsServer;
+pub use self::server::TlsListener;
+#[cfg(unix)]
+pub use self::server::UnixListener;
+pub use self::server::{JoinedListener, Server, TcpListener};
 pub use self::service::Service;
 pub use self::writer::Writer;
 pub use async_trait::async_trait;
@@ -49,11 +46,12 @@ pub mod prelude {
     pub use crate::depot::Depot;
     pub use crate::http::errors::*;
     pub use crate::http::{Request, Response, StatusCode};
-    pub use crate::routing::filter;
-    pub use crate::routing::Router;
-    pub use crate::server::Server;
+    pub use crate::routing::{filter, Router};
     #[cfg(feature = "tls")]
-    pub use crate::server::TlsServer;
+    pub use crate::server::TlsListener;
+    #[cfg(unix)]
+    pub use crate::server::UnixListener;
+    pub use crate::server::{JoinedListener, Server, TcpListener};
     pub use crate::service::Service;
     pub use crate::writer::*;
     pub use crate::Handler;
@@ -82,7 +80,9 @@ fn new_runtime(threads: usize) -> Runtime {
 ///     "Hello World"
 /// }
 /// fn main() {
-///    let server = Server::new(Router::new().get(hello_world)).bind(([0, 0, 0, 0], 7878));
+///
+///    let service = Service::new(Router::new().get(hello_world));
+///    let server = Server::bind(&"127.0.0.1:7878".parse().unwrap()).serve(service);
 ///    salvo_core::start(server);
 /// }
 /// ```
@@ -99,7 +99,8 @@ pub fn start<F: Future>(future: F) {
 ///     "Hello World"
 /// }
 /// fn main() {
-///    let server = Server::new(Router::new().get(hello_world)).bind(([0, 0, 0, 0], 7878));
+///    let service = Service::new(Router::new().get(hello_world));
+///    let server = Server::bind(&"127.0.0.1:7878".parse().unwrap()).serve(service);
 ///    salvo_core::start_with_threads(server, 8);
 /// }
 /// ```
